
import java.lang.*;
import java.awt.*;
import java.awt.image.*;

public class CMUcam extends java.lang.Object
{
    public TypeC dataC;
    public TypeM trackOnce;
    public TypeS mean;

    private SerialPort serialPort;
    private TrikebotController trikebot;
    private NetPort dumpPort;
    private boolean serial;
    private boolean pollmode = false;

    private boolean RAWP = false;
    private final static int READ_TIMEOUT = 2000;

    private GrabThread dumpThr;
    public CamCanvas pic;
  
    public final static int VIEWFIELD = 40;
    
    /* Here is the CMUcam constructor. Use this to instantiate a camera */
    public CMUcam()
	{
            mean = new TypeS();
            dataC = new TypeC();
            trackOnce = new TypeM();
            serialPort = null;
            trikebot = null;
            dumpPort = null;

            pic = new CamCanvas(80*4, 143*2);
    }

    /* Port should be the COMX number.
       The rate should be the baud rate for CMUcam, which is normally
       115200 which is rate = 5.  Note that if this
       fails to open the serial port you get a false return value
       so you should DEFINITELY check for this in your code */
    public boolean InitCamera(int port, int rate)
	{
        int error;
        serial = true;
        serialPort = new SerialPort();
        
        error = serialPort.openSerial0(port, rate);
        if (error != 0)
		{
            System.err.println("Error " + error + " in camera openSerial()");
            return false;
		}
		else
		{
            SetReadTimeout(READ_TIMEOUT);
            System.err.println("camera serial successfully opened!");

            return true;
         }
    }

    /* TCP/IP (wireless) initCamera version */
    public boolean InitCamera(String ipaddr, TrikebotController t)
	{
        int error;
        serial = false;

        dumpPort = new NetPort();

        trikebot = t;
        
        error = dumpPort.OpenSocket(ipaddr, 5001);
        if (error != 0)
		{
            System.err.println("Error " + error + " in initCamera calling OpenSocket for dumpframe");
            return false;
        }
        System.err.println("Camera dumpframe socket successfully opened!");

        SetReadTimeout(READ_TIMEOUT);
        
        return true;
    }

    public boolean ClosePorts()
	{
        int error;
        if (serial && serialPort != null)
		{
            error = serialPort.closeSerial0();
            if (error != 0)
			{
                System.err.println("Error closing serial port!");
                return false;
            }
        }
		else
		{
			if (dumpPort != null)
			{
                error = dumpPort.CloseSocket();
                if (error != 0)
				{
                    System.err.println("Error closing dumpFrame socket (5001)!");
                    return false;
                }
            }
        }
        return true;
    }
    
    public void SetReadTimeout(int timeoutval)
	{
        if (serial)
            serialPort.setReadTimeout0(timeoutval);
        else
			dumpPort.SetReadTimeout(timeoutval);
    }

    /* ------------------------------ */
    /* CAMERA INTERFACE COMMANDS      */
    /*  -this is what you use!        */
    /* ------------------------------ */

    // returns true if the camera acknowledges messages //
    public boolean CheckCamera()
	{
        return CommandNoResponse(" ");
    }

    /*
	For our class, this sets RGB mode, auto white balance on,
	autogain on, mass mode enabled, poll mode enabled (IMPORTANT),
	noise filter on, full window size
	WARNING: if this returns false, call it again!
	*/
    public boolean ResetCamera()
	{
		if (!CommandNoResponse("rs")) return false;
		// rgb mode, auto gain and white balance on
		if (!CommandNoResponse("cr 18 44 19 33")) return false;
		// mass mode on
		if (!CommandNoResponse("mm 1")) return false;
		// poll mode on
		if (!CommandNoResponse("pm 1")) return false;
		pollmode = true;
		
		// noise filter on
		if (!CommandNoResponse("nf 1")) return false;
		// set full window size (no subwindowing)
		if (!CommandNoResponse("sw")) return false;
		// if desired, enter Raw Mode (use flag to select)
		if (RAWP)
			if (!CommandNoResponse("rm 1")) return false;
		
		return true;
    }
	
	public boolean SetContrast(int contrast)
	{
		return CommandNoResponse("cr 5 "+contrast);
	}
	
	public boolean SetBrightness(int brightness)
	{
		return CommandNoResponse("cr 6 "+brightness);
	}
	
	public boolean SetAutoWhiteBalance(boolean on)
	{
		if (on)
			return CommandNoResponse("cr 18 44");
		else
			return CommandNoResponse("cr 18 40");
	}
	
	public boolean SetAutoGain(boolean on)
	{
		if (on)
			return CommandNoResponse("cr 19 33");
		else
			return CommandNoResponse("cr 19 32");
	}

    // Sets the window for all functions. Remember, maximum
    // width is 80 and height is 143
    // so full screen is setWindow(1,1,80,143) //
    public boolean SetWindow(int x1, int y1, int x2, int y2)
	{
        return CommandNoResponse("sw " + x1 + " " + y1 + " " + x2 + " " + y2);
    }

    // returns a string from the camera responding to get version
    public String GetVersion()
	{
        return CommandStringResponse("gv");
    }

    // returns true if mean is updated with new statistics from camera
    // assumes poll mode is on!!!!!
    public boolean GetMean()
	{
        return CommandTypeS("gm", mean);
    }

    // NOTE: trackColor assumes that MassMode is on! Otherwise, write
    // your own function to call CommandTypeC instead.
    // returns true if DataM is updated by camera
    public boolean TrackColor(int rmin, int rmax, int gmin, int gmax, int bmin, int bmax)
	{
         return CommandTypeM("tc "+rmin+" "+rmax+" "+gmin+" "+gmax+" "+bmin+" "+bmax, trackOnce);
    }

	public boolean TrackWindow()
	{
		return CommandTypeSM("tw", mean, trackOnce);
    }

    //This initializes frameMem then grabs a frame -many frames' columns
    // actually- and populates the part only set by setwindow
    // frameMem[i][j] is read as follows. i is the column number,
    // j is the rgb values of the rows, so frameMem[3][9], [3][10], [3][11]
    // would be the rgb values of pixel (3,3)
    // now the tricky thing. The origin for frameMem, [0][0],
    // corresponds to the bottom left of the image I think
    // tricky thing: the parameters are based on origin at top right
    // in 16x62 because camera is upside-down. Otherwise it's bottom left
    public boolean GrabFrame()
	{
        return GrabFrame(1, 1, 80, 143); // The whole window
	}	
	
    public boolean GrabFrame(int winx1, int winy1, int winx2, int winy2)
	{
        if (serial)
            dumpThr = new GrabThread(this, serialPort);
        else
            dumpThr = new GrabThread(this, trikebot, dumpPort);

		SetWindow(winx1, winy1, winx2, winy2);
        dumpThr.setWin(winx1, winy1, winx2, winy2);
        dumpThr.start();
        return true;
	}
	
    public boolean GrabFrameAndWait()
	{
        return WaitForGrabFrame(GrabFrame()); // The whole window
	}

    public boolean GrabFrameAndWait(int winx1, int winy1, int winx2, int winy2)
	{
        return WaitForGrabFrame(GrabFrame(winx1, winy1, winx2, winy2)); // The whole window
	}

	public void WaitForGrabFrame()
	{
		Thread.yield();
		synchronized (dumpPort) {}
	}

	private boolean WaitForGrabFrame(boolean b)
	{
		WaitForGrabFrame();
		return b;
	}
	
	public Image Image()
	{
		return pic.Image();
	}

	public Image WaitForImage()
	{
		WaitForGrabFrame();
		return pic.Image();
	}

	public void GrabFrameAndKeep(String title)
	{
		GrabFrame();
		new ImageWindow(WaitForImage(), title);
	}

    /* ------------------------------    */
    /* CAMERA INTERFACE HELPER COMMANDS  */
    /* ------------------------------    */

    // used for all commands that expect nothing but an ACK back
    // returns true if the command got acknowledged by the camera
    public boolean CommandNoResponse(String cmdString)
	{
        /* CR, NF, PM, SW, MM  */
        String rsp = CommandStringResponse(cmdString);
        return (rsp != null);
    }

    /* sends the command and returns the response string, stripped of ACK
     * and the final "\r:". if NCK or timeout occurs, returns null.
     * Doesn't work for raw mode!
     */
    public String CommandStringResponse(String cmdString)
	{
        ClearBuffer();
        return ParseResponse(SendString(cmdString));
    }

    // For any command that returns a type S packet
    // That means GetMean basically
    public boolean CommandTypeS(String cmdString, TypeS packetS)
	{
        // GM -- WARNING: we assume poll mode here!
        // packetS should already be instantiated
        String returnStr;
        boolean success = false;
        ClearBuffer();
        if (RAWP)
		{
			returnStr = ParseResponse(SendRawLine(cmdString));
            
            if (returnStr != null)
                success = packetS.rawParseString(returnStr);
        }
		else
		{
            returnStr = ParseResponse(SendString(cmdString));
            
            if (returnStr != null)
                success = packetS.parseString(returnStr);
        }

        return success;
    }

    // For any command that returns TypeC data //
    // That would be TrackColor when MassMode is off (which never happens) //
    public boolean CommandTypeC(String cmdString, TypeC packetC)
	{
        // TC -- WARNING: we assume poll mode here!
        // packetC should already be instantiated  //
        String returnStr;
        boolean success = false;
        ClearBuffer();
        if (RAWP)
		{
            returnStr = ParseResponse(SendRawLine(cmdString));
            
            if (returnStr != null)
                success = packetC.rawParseString(returnStr);
        }
		else
		{
            returnStr = ParseResponse(SendString(cmdString));
            
            if (returnStr != null)
                success = packetC.parseString(returnStr);
        }

        return success;
    }

    // For any command that returns TypeM data
    // That would be TrackColor when MassMode is on (always by default)
    public boolean CommandTypeM(String cmdString, TypeM packetM)
	{
        // TC -- WARNING: we assume poll mode here!
        // packetM should already be instantiated
        String returnStr;
        boolean success = false;
        ClearBuffer();
        if (RAWP)
		{
            returnStr = ParseResponse(SendRawLine(cmdString));
            
            if (returnStr != null)
                success = packetM.rawParseString(returnStr);
        }
		else
		{
            returnStr = ParseResponse(SendString(cmdString));
            
            if (returnStr != null)
                success = packetM.parseString(returnStr);
        }

		trikebot.track.ParseTypeM(packetM);
		
        return success;
    }

    // For any command that returns an S and M packet
    // That would be TrackWindow when MassMode is on (always by default)
    public boolean CommandTypeSM(String cmdString, TypeS packetS, TypeM packetM)
	{
        // TC -- WARNING: we assume poll mode here!
        // packetM should already be instantiated
        String returnStr;
        boolean success = false;
        ClearBuffer();
        if (RAWP)
		{
            returnStr = ParseResponse(SendRawLine(cmdString));

            if (returnStr != null)
                success = packetS.rawParseString(returnStr.substring(0, returnStr.indexOf('M')-1))
					&& packetM.rawParseString(returnStr.substring(returnStr.indexOf('M')));
        }
		else
		{
            returnStr = ParseResponse(SendString(cmdString));

            if (returnStr != null)
                success = packetS.parseString(returnStr.substring(0, returnStr.indexOf('M')-1))
					&& packetM.parseString(returnStr.substring(returnStr.indexOf('M')));
        }

		trikebot.track.ParseTypeM(packetM);

        return success;
    }

    /* ------------------------------ */
    /* COMMUNICATION HELPER FUNCTIONS */
    /* ------------------------------ */
    
    public void ClearBuffer()
	{
		// System.err.println("Entering ClearBuffer..");
        
        if (serial)
		{
            SetReadTimeout(10);
            while (true)
			{
                int value = serialPort.readByte0();
                if (value == -1)
                    break;
            }
            SetReadTimeout(READ_TIMEOUT);
        }
		else
		{
            //camPort.flush();
        }
    }

    // Send a string as a series of bytes
    private String SendString(String s)
	{
//		System.err.println("Entering CMUcam SendString...");
        if (serial)
		{
			/*
            byte[] bytes = s.getBytes();
			int i, error;
            for (i = 0; i < bytes.length; i++)
			{
                error = serialPort.sendByte0(ByteUtil.Unsign(bytes[i]));
                if (error != 0)
				{
                    break;
                }
            }
            
//			System.err.println("sent " + i + " bytes of: " + s);
            return i;
			*/
			return null;
        }
		else
		{
            return trikebot.CMUcamDialogue(s);
        }
    }

    private String SendLine(String s)
	{
        //System.err.println("CMUcam: SendLine: sending "+s);
        return SendString(s + "\r");
    }

    private String SendRawLine(String s)
	{
        return SendString(s);
    }
    
    /* reads from the camera until it receives the "\r:". checks for ACK or NCK.
     * in the case of NCK, returns null. in the case of ACK, returns anything
     * between the ACK and the "\r:".
     */
    private String ParseResponse(String s)
	{
        if (s == null)
		{
			System.err.println("Null string");
            return null;
		}
		System.err.println("Response string: '"+s+"'");
        if (!s.startsWith("ACK\r"))
            return null;
        
        int l = s.lastIndexOf("\r:");
        if (l < 4)
            return "";
        return s.substring(4, l);  // strip "ACK\r" from front and "\r:" from end
    }
}