//
//  Calibration.java
//  Trikebot
//
//  Created by Andrs Santiago Prez-Bergquist on Mon Jul 08 2002.
//  Copyright (c) 2002. All rights reserved.
//

import java.lang.*;
import java.io.*;

public class Calibration
{
    public static String GenCode(CalibrationConstants constants)
	{
		return
		"//\r"+
		"// CalibrationConstants.java\r"+
		"// Trikebot\r"+
		"//\r"+
		"// Created by Andrs Santiago Prez-Bergquist on Mon Jul 08 2002.\r"+
		"// Copyright (c) 2002. All rights reserved.\r"+
		"//\r"+
		"\r"+
		"public class CalibrationConstants\r"+
		"{\r"+
		"\t// Calibration min/max trikebot parameters\r"+
		"\tpublic int PANMIN = "+constants.PANMIN+";\r"+
		"\tpublic int PANMAX = "+constants.PANMAX+";\r"+
		"\t\r"+
		"\tpublic int PANMIN_ANGLE = "+constants.PANMIN_ANGLE+";\r"+
		"\tpublic int PANMAX_ANGLE = "+constants.PANMAX_ANGLE+";\r"+
		"\t\r"+
		"\tpublic int TILTMIN = "+constants.TILTMIN+";\r"+
		"\tpublic int TILTMAX = "+constants.TILTMAX+";\r"+
		"\t\r"+
		"\tpublic int TILTMIN_ANGLE = "+constants.TILTMIN_ANGLE+";\r"+
		"\tpublic int TILTMAX_ANGLE = "+constants.TILTMAX_ANGLE+";\r"+
		"\t\r"+
		"\tpublic int TURNMIN = "+constants.TURNMIN+";\r"+
		"\tpublic int TURNMAX = "+constants.TURNMAX+";\r"+
		"\t\r"+
		"\tpublic int TURNMIN_ANGLE = "+constants.TURNMIN_ANGLE+";\r"+
		"\tpublic int TURNMAX_ANGLE = "+constants.TURNMAX_ANGLE+";\r"+
		"\t\r"+
		"\tpublic int WHEEL_FWD_MIN = "+constants.WHEEL_FWD_MIN+";\r"+
		"\tpublic int WHEEL_FWD_MAX = "+constants.WHEEL_FWD_MAX+";\r"+
		"\t\r"+
		"\tpublic int WHEEL_REV_MIN = "+constants.WHEEL_REV_MIN+";\r"+
		"\tpublic int WHEEL_REV_MAX = "+constants.WHEEL_REV_MAX+";\r"+
		"\t\r"+
		"\tpublic int WHEEL_REV_SPEED = "+constants.WHEEL_REV_SPEED+";\r"+
		"\tpublic int WHEEL_FWD_SPEED = "+constants.WHEEL_FWD_SPEED+";\r"+
		"}";
	}

	public static boolean SaveCode(CalibrationConstants constants, File destFile)
	{
		try
		{
			FileWriter dest = new FileWriter(destFile);
			dest.write(GenCode(constants));
			dest.close();
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}

	public static boolean SaveDefault(CalibrationConstants constants)
	{
		return SaveCode(constants, new File("CalibrationConstants.java"));
	}
	public static boolean OpenCode(CalibrationConstants constants, File srcFile)
	{
		try
		{
			int[] constantsArray = new int[18];

			FileReader src= new FileReader(srcFile);
			char c = 0;
			for (int i = 0; i < 18; i++)
			{
				while(c != '=')			// Read up to the equals sign
					c = (char) src.read();
				c = (char) src.read();	// Get the space after the equals sign
				c = (char) src.read();	// Get the first digit of the number
				String s = new String();
				while(c != ';')			// Read up until the end of the line
				{
					s += c;
					c = (char) src.read();
				}
				System.out.println("'"+s+"'");
				constantsArray[i] = Integer.parseInt(s);
				System.out.println(constantsArray[i]);
			}
			src.close();

			constants.PANMIN = constantsArray[0];
			constants.PANMAX = constantsArray[1];

			constants.PANMIN_ANGLE = constantsArray[2];
			constants.PANMAX_ANGLE = constantsArray[3];

			constants.TILTMIN = constantsArray[4];
			constants.TILTMAX = constantsArray[5];

			constants.TILTMIN_ANGLE = constantsArray[6];
			constants.TILTMAX_ANGLE = constantsArray[7];

			constants.TURNMIN = constantsArray[8];
			constants.TURNMAX = constantsArray[9];

			constants.TURNMIN_ANGLE = constantsArray[10];
			constants.TURNMAX_ANGLE = constantsArray[11];

			constants.WHEEL_FWD_MIN = constantsArray[12];
			constants.WHEEL_FWD_MAX = constantsArray[13];

			constants.WHEEL_REV_MIN = constantsArray[14];
			constants.WHEEL_REV_MAX = constantsArray[15];

			constants.WHEEL_REV_SPEED = constantsArray[16];
			constants.WHEEL_FWD_SPEED = constantsArray[17];
			
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}
}

