/*
 * CamCanvas.java
 *
 * Created on May 28, 2002, 10:52 AM
 */


import java.awt.*;
import java.awt.image.*;

public class CamCanvas extends Canvas
{
    private Image myImg;
    private Image offScreenImage;
	private Dimension size;
    private Dimension offScreenSize;
    private Graphics offScreenGraphics;
	private int[] pixels;
    
    /** Creates new CamCanvas */
    public CamCanvas(int w, int h)
	{
        this.setSize(w, h);
        
        pixels = new int[80*143];
        for (int i = 0; i < 80*143; i++)
            pixels[i] = (Color.white).getRGB();
        myImg = (createImage (new MemoryImageSource(80, 143, pixels, 0, 80))).getScaledInstance(w, h, Image.SCALE_SMOOTH);
    }

	public Dimension getMinimumSize()
	{
		return size;
	}

	public Dimension getPreferredSize()
	{
		return size;
	}

	public Dimension getMaximumSize()
	{
		return size;
	}

	public void setSize(int width, int height)
	{
		size = new Dimension(width, height);
		//super.setSize(size);
	}

	public void setSize(Dimension d)
	{
		size = d;
		//super.setSize(size);
	}
    
    public void paint(Graphics g)
	{
        g.drawImage(myImg, 0, 0, null);
    }

    public final synchronized void update(Graphics g)
	{
        Dimension d = getSize();
        if ((offScreenImage == null) || (d.width != offScreenSize.width) || (d.height != offScreenSize.height))
		{
            offScreenImage = createImage(d.width, d.height);
            offScreenSize = d;
            offScreenGraphics = offScreenImage.getGraphics();
        }
        offScreenGraphics.clearRect(0, 0, d.width, d.height);
        paint(offScreenGraphics);
        g.drawImage(offScreenImage, 0, 0, null);
    }

    private void changeImg(MemoryImageSource newImgSource)
	{
        Image newImg = createImage(newImgSource);
        myImg = newImg.getScaledInstance(this.getWidth(), this.getHeight(), Image.SCALE_SMOOTH);

        update(this.getGraphics());
    }

    public Image Image()
	{
        return myImg;
    }

	public int[] Pixels()
	{
		return pixels;
	}

	public void SetPixels(int[] newPixels)
	{
		pixels = newPixels;
		changeImg(new MemoryImageSource(80, 143, pixels, 0, 80));
	}

	public Color ColorAt(int x, int y)
	{
		return new Color(pixels[80*(143-y) + (x-1)]);
	}

	public int RedAt(int x, int y)
	{
		return ColorAt(x, y).getRed();
	}

	public int GreenAt(int x, int y)
	{
		return ColorAt(x, y).getGreen();
	}

	public int BlueAt(int x, int y)
	{
		return ColorAt(x, y).getBlue();
	}
}