/*
 * GrabThread.java
 *
 * Created on May 14, 2002, 4:58 PM
 */

import java.awt.*;
import java.awt.image.*;
import java.util.*;

public class GrabThread extends Thread
{
    private TrikebotController trikebot;
    private NetPort dumpPort;
    private SerialPort serialPort;
    private CMUcam theCam;
    private int winx1, winx2;
    private int winy1, winy2;
    
    private boolean serial;
    
    private int[] pixels;
	
	private final int LONG_TIMEOUT = 10000;
	private final int SHORT_TIMEOUT = 1000;	
    
    /** Creates new GrabThread */
    public GrabThread(CMUcam parent, TrikebotController t, NetPort dnet)
	{
        serial = false;
        trikebot = t;
        dumpPort = dnet;
        theCam = parent;
        pixels = new int[80*143];
        
        winx1 = 1;
        winy1 = 1;
        winx2 = 80;
        winy2 = 143;
    }

    public GrabThread(CMUcam parent, SerialPort port)
	{
        serial = true;
        serialPort = port;
        theCam = parent;
        pixels = new int[80*143];
        
        winx1 = 1;
        winy1 = 1;
        winx2 = 80;
        winy2 = 143;
    }

    public void setWin(int x1, int y1, int x2, int y2)
	{
        winx1 = x1;
        winy1 = y1;
        winx2 = x2;
        winy2 = y2;
    }
    
    public void run()
	{
        System.err.println("Entering dumpframe thread!");
        grabFrame();
    }
    
    public boolean grabFrame()
	{
		int value, row, col;
		int cstart = winx2 - 1; int rstart = 143 - winy2;
		row = rstart; col = cstart;

		for (int i=0; i < 80*143; i++)
			pixels[i] = (Color.black).getRGB();
		
		synchronized(dumpPort)
		{
			if (serial)
			{
				byte[] bytes = "DF\r".getBytes();
				for (int i = 0; i < bytes.length; i++)
				{
					int error = serialPort.sendByte0(ByteUtil.Unsign(bytes[i]));
					if (error != 0)
						break;
				}
			}
			else
			{
				dumpPort.Flush();
				dumpPort.SetReadTimeout(LONG_TIMEOUT);
				trikebot.SendDumpFramePacket();
			}
			
			if (!detectAck())
			{
				System.err.println("Grab Frame: no ACK received!");
				
				if (!serial)
				{
					//camPort.SendString("STOPDF");
					//camPort.Flush();
					dumpPort.Flush();
				}
				
				return false;
			}

			boolean done = false;
			boolean error = false;
			while (!done && !error)
			{
				value = readDumpByte();
				//System.err.println("Dump byte: "+value);
				switch (value)
				{
					case -1:
						System.err.println("ERROR: timeout during frame grab");
						error = true;
						break;
					case 1:
						System.err.println("1");
						break;
					case 2:
						System.err.println("2 " + col + " " + row);
						col--;
						row = rstart;
						if (col % 10 == 0)  // refresh after every ten columns
							theCam.pic.SetPixels(pixels);
						break;
					case 3:
						System.err.println("3");
						done = true;
						break;
					default:
						int red, green, blue;
					
						red = value;
						green = readDumpByte();
						//System.err.println("Dump byte: "+green);
						blue = readDumpByte();
						//System.err.println("Dump byte: "+blue);
						
						if (green == -1 || blue == -1)
						{
							System.err.println("ERROR: timeout during frame grab");
							error = true;
							break;
						}
						
						if (col < 0 || row >= 143)
						{
							System.err.println("ERROR: must have missed a 2 during frame grab!");
							error = true;
							break;
						}
						
						pixels[row*80 + col] = (new Color(red, green, blue)).getRGB();
						row++;
				}
			}
			// Output the final version
			theCam.pic.SetPixels(pixels);

			if (error)
			{
				Purge();
				return false;
			}
			else
			{
				toColon();
				return true;
			}
		}
    }

    private int readDumpByte()
	{
        if (serial)
            return serialPort.readByte0();
        else
            return dumpPort.ReadRawByte();
    }
    
    private void toColon()
	{
        int value;
        value = readDumpByte();
        while (value != -1 && value != ':')
            value = readDumpByte();
        
        if (value == -1)
            System.err.println("toColon: timeout!");
    }

    private boolean detectAck()
	{
        String s = "";
//		System.err.println("CMUcam: entering detectAck");

        int value = readDumpByte();
		dumpPort.SetReadTimeout(SHORT_TIMEOUT);
        while (value != 13)
		{
			//System.err.println("detectACK: "+value);
            if (value == -1)
			{
                System.err.println("CMUcam: ReadLine timeout; partial packet: "+s);
                break;
            }
			s += (char) value;
			value = readDumpByte();
        }

//		System.err.println("detectACK: " + s);

        return s.startsWith("ACK");
    }

	private void Purge()
	{
		dumpPort.SetReadTimeout(SHORT_TIMEOUT*2);
		while (readDumpByte() != -1);
	}
}
