//
//  JoystickCanvas.java
//  Trikebot
//
//  Created by Andrs Santiago Prez-Bergquist on Wed Jun 12 2002.
//  Copyright (c) 2002. All rights reserved.
//

import java.awt.*;
import java.lang.*;

public class JoystickCanvas extends Canvas
{
	int x, y;						// Current x & y in local screen coordinates, with y flipped vertically
	int sizeX, sizeY;				// Width and height in pixels
	int centerX, centerY;			// Origin in pixels
	float xLeft, xRight, yTop, yBottom;	// Range of axes in logical coordinates
	
	public JoystickCanvas(int width, int height, float xMin, float xMax, float yMax, float yMin)
	{
		setSize(width, height);
		SetAxes(xMin, xMax, yMax, yMin);
		Recenter();
	}

	// Resize to so many pixels
	public void setSize(int width, int height)
	{
		sizeX = width;
		sizeY = height;
		//super.setSize(sizeX, sizeY);
	}

	public void setSize(Dimension d)
	{
		sizeX = d.width;
		sizeY = d.height;
		//super.setSize(sizeX, sizeY);
	}

	public Dimension getMinimumSize()
	{
		return new Dimension(sizeX, sizeY);
	}

	public Dimension getPreferredSize()
	{
		return new Dimension(sizeX, sizeY);
	}

	public Dimension getMaximumSize()
	{
		return new Dimension(sizeX, sizeY);
	}

	public void SetAxes(float xMin, float xMax, float yMax, float yMin)
	{
		xLeft	= xMin;
		xRight	= xMax;
		yTop	= yMin;
		yBottom	= yMax;
		ComputeCenter();
	}

	// Find the center point after a setSize
	private void ComputeCenter()
	{
		centerX = (int) (sizeX * (xLeft / (xLeft - xRight)));
		centerY = (int) (sizeY * (yTop / (yTop - yBottom)));
	}

	// Input in screen coordinates
	public void SetPos(int X, int Y)
	{
		x = X;
		if (x < 0)     x = 0;
		if (x > sizeX) x = sizeX;
		y = Y;
		if (y < 0)     y = 0;
		if (y > sizeY) y = sizeY;
		repaint();
	}

	// Input in logical coordinates
	public void SetLogical(int X, int Y)
	{
		x = (int) (((X - xLeft) * sizeX) / (xRight - xLeft));
		y = (int) (((Y - yTop ) * sizeY) / (yBottom - yTop));
		SetPos(x, y);
	}

	// Input in logical coordinates
	public void SetLogicalX(int X)
	{
		x = (int) (((X - xLeft) * sizeX) / (xRight - xLeft));
		SetPos(x, y);
	}

	// Input in logical coordinates
	public void SetLogicalY(int Y)
	{
		y = (int) (((Y - yTop ) * sizeY) / (yBottom - yTop));
		SetPos(x, y);
	}

	// Set pointer to origin
	public void Recenter()
	{
		x = centerX;
		y = centerY;
		repaint();
	}

	// Current (x, y) in logical coordinates, not screen coordinates
	public float X()
	{
		return xLeft + ((xRight - xLeft) * x) / sizeX;
	}

	public float Y()
	{
		return yTop + ((yBottom - yTop) * y) / sizeY;
	}
	
	public void paint(Graphics g)
	{
		g.setColor(Color.white);
		g.fillRect(0, 0, sizeX, sizeY);
		g.setColor(Color.black);
		g.drawRect(0, 0, sizeX-1, sizeY-1);

		g.setColor(Color.blue);
		g.drawLine(0, centerY, sizeX, centerY);
		g.drawLine(centerX, 0, centerX, sizeY);

		g.setColor(Color.red);
		g.fillOval(x-2, y-2, 5, 5);
	}
}