//
//  Macros.java
//  Trikebot
//
//  Created by Andrs Santiago Prez-Bergquist on Mon Jul 22 2002.
//  Copyright (c) 2002. All rights reserved.
//

import java.io.*;
import java.util.*;

public class Macros extends java.lang.Object
{
	NetPort	controlPort;
	TreeMap	macros;
	ByteArrayOutputStream	storage;

	public Macros()
	{
		SetPort(null);
		macros = new TreeMap();
	}

	public Macros(NetPort port)
	{
		SetPort(port);
		macros = new TreeMap();
	}

	public void SetPort(NetPort port)
	{
		controlPort = port;
	}

	public String[] Defined()
	{
		return (String[]) macros.keySet().toArray(new String[0]);
	}

	public boolean StartRecording()
	{
		storage = new ByteArrayOutputStream();
		return controlPort.StartLogging(storage);
	}

	public boolean StopRecording(String macroName)
	{
		boolean result = controlPort.StopLogging();
		macros.put(macroName, storage.toByteArray());
		return result;
	}

	public boolean CancelRecording()
	{
		return controlPort.StopLogging();
	}
	
	public boolean PlayBack(String macroName)
	{
		return controlPort.ReplayLog(new ByteArrayInputStream((byte[]) macros.get(macroName)));
	}

	public boolean Save(String macroName, File file)
	{
		try
		{
			FileOutputStream out = new FileOutputStream(file);
			out.write((byte[]) macros.get(macroName));
			out.close();
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}

	public boolean Open(String macroName, File file)
	{
		try
		{
			FileInputStream in = new FileInputStream(file);
			int length = in.available();
			byte[] b = new byte[length];
			in.read(b, 0, length);
			in.close();
			macros.put(macroName, b);
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}

	public boolean Save(String macroName, String fileName)
	{
		return Save(macroName, new File(fileName));
	}

	public boolean Open(String macroName, String fileName)
	{
		return Open(macroName, new File(fileName));
	}
}
