
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class NetPort extends java.lang.Object
{
	private final static boolean DEBUG = false;			// Display packets sent and received
	private final static boolean DEBUG_MORE = false;	// Display every byte read and written
	
    private Socket theSocket = null;
    private OutputStream outStream = null;
    private InputStream inStream = null;
    private int readTimeout = 500;

	private long time = 0;
	private boolean logging = false;
	OutputStream log;

	public final byte   control = (byte) 255;
    public final byte[] quit 	= {control, (byte) 54};
    public final byte[] parse	= {control, (byte) 55};
    public final byte[] flush	= {control, (byte) 56};
    
    public int OpenSocket(String ipaddr, int portnum)
	{
        System.err.println("in OpenSocket; trying " + ipaddr + " " + portnum);
        
        try
		{
            theSocket = new Socket(ipaddr, portnum);
            outStream = theSocket.getOutputStream();
            inStream = theSocket.getInputStream();
        }
		catch (UnknownHostException e)
		{
            System.err.println("Cannot find host in OpenSocket!");
            return -1;
        }
		catch (IOException e)
		{
            System.err.println("IO exception: socket");
            return -1;
        }
		/*
		try { System.err.println("Buffer size: "+theSocket.getReceiveBufferSize()); } catch (Exception e) {}
		try { theSocket.setReceiveBufferSize(40000); } catch (Exception e) {}
		try { System.err.println("Buffer size: "+theSocket.getReceiveBufferSize()); } catch (Exception e) {}
		 */
        return 0;                
    }

    public int CloseSocket()
	{
        try
		{
            if (inStream != null)
                inStream.close();
            if (outStream != null)
                outStream.close();
            if (theSocket != null)
                theSocket.close();
        }
		catch (IOException e)
		{
            System.err.println("IO exception closing socket!");
            return -1;
        }
        return 0;
    }

    public int SendBytes(byte[] bytes)
	{		
		synchronized (outStream)
		{
			if (logging)
			{
				long newTime = Time.Current();
				long timeDiff = newTime - time;
				try
				{
					log.write(ByteUtil.SplitLong(timeDiff));
					log.write((byte) bytes.length);
					log.write(bytes);
				}
				catch (Exception e)
				{}
				time = newTime;
			}
			
			if (DEBUG)
			{
				System.err.println("SendBytes (Text): '"+new String(bytes)+"'");
				String debug = "SendBytes (Raw):";
				for (int i = 0; i < bytes.length; i++)
					debug += (" " + ByteUtil.Unsign(bytes[i]));
				System.err.println(debug);
			}
			
			if (outStream == null)
			{
				System.err.println("call OpenSocket first!");
				return -1;
			}
	
			if (DEBUG_MORE)
				System.err.println("Writing to socket: " + theSocket.toString());
			
			try
			{
				if (DEBUG_MORE)
					System.err.println("Sending Flush");
				outStream.write(flush);
				for (int i = 0; i < bytes.length; i++)
				{
					if (DEBUG_MORE)
						System.err.println("Sending "+ByteUtil.Unsign(bytes[i]));
					outStream.write(bytes[i]);
					if (bytes[i] == control)
						outStream.write(255);
				}
				if (DEBUG_MORE)
					System.err.println("Sending Parse");
				outStream.write(parse);
				outStream.flush();
			}
			catch (IOException e)
			{
				System.err.println("outStream.write: IO exception " + e);
				return -1;
			}
			return 0;
		}
	}
    
    public int SendString(String s)
	{
        int i;
        byte[] bytes = s.getBytes();
        return SendBytes(bytes);
    }

    public int SendLine(String s)
	{
        //System.err.println("Printing the string: " + s);
        return SendString(s + "\r");
    }
	/*
	private int SendByte(int b)
	{
		if (outStream == null)
		{
			System.err.println("call OpenSocket first!");
			return -1;
		}
		
		try
		{
			outStream.write(b);
			if ((byte)b == control)
				outStream.write(b);
		}
		catch (IOException e)
		{
			System.err.println("In SendByte: IO Exception");
		}
		return 0;
	}
	 */
	public int SendRawByte(int b)
	{
		synchronized (outStream)
		{
			if (outStream == null)
			{
				System.err.println("call OpenSocket first!");
				return -1;
			}

			try
			{
			outStream.write(b);
			}
			catch (IOException e)
			{
				System.err.println("In SendByte: IO Exception");
			}
			return 0;
		}
	}

    public String ReadLine()
	{
        return new String(ReadByteLine());
    }

    public byte[] ReadByteLine()
	{
		synchronized (inStream)
		{		
			Vector v = new Vector();
			
			while (true)
			{
				int b = ReadByte();
				
				if (b == -1)
				{
					System.err.println("Error in NetPort.ReadByteLine(): timeout!");
					break;
				}
				
				if ((byte)b == control)
				{
					b = ReadByte();
					if ((byte)b == parse[1])
					{
						break;  // end of packet
					}
					else if ((byte)b == flush[1])
					{
						v = new Vector();
						continue; // Don't add the 56 to the buffer
					}
					else if (b == -1)
					{
						System.err.println("Error: timeout after receiving 255!");
						break;
					}
				}
	
				v.add(new Byte((byte)b));
			}
	
			byte[] bytes = new byte[v.size()];
			String debug = "ReadByteLine (Raw):";
			for (int i = 0; i < v.size(); i++)
			{
				bytes[i] = ((Byte)(v.elementAt(i))).byteValue();
				if (DEBUG)
					debug += (" " + ByteUtil.Unsign(bytes[i]));
			}
			if (DEBUG)
			{
				System.err.println("ReadByteLine (Text): '"+(new String(bytes))+"'");
				System.err.println(debug);
			}
			
			return bytes;
		}
	}

	// Synchronized for thread-safety
    public int ReadRawByte()
	{
		synchronized (inStream)
		{
			return ReadByte();
		}
	}

    // this is the complex one. This implements the timeout
	// No need for synchronization here, since this is for internal use only
    private int ReadByte()
	{
        byte[] input_byte = new byte[1];

        if (inStream == null)
		{
            System.err.println("call OpenSocket first!");
            return -1;
        }
       
        try
		{
            if (inStream.available() > 0)
			{
                inStream.read(input_byte);
				if (DEBUG_MORE)
					System.err.println("ReadByte: "+ByteUtil.Unsign(input_byte[0]));            
                return ByteUtil.Unsign(input_byte[0]);
            }
			else
			/*
			{ // now block until timeout //
                Date curdate = new Date();
                long start_time = curdate.getTime();
                long cur_time = start_time;
                while ((cur_time - start_time) < readTimeout)
				{
                    Time.Sleep(10);
                    if (inStream.available() > 0)
					{
                        inStream.read(input_byte);
                        if (DEBUG_MORE)
							System.err.println("ReadByte: "+ByteUtil.Unsign(input_byte[0]));                                
                        return ByteUtil.Unsign(input_byte[0]);
                    }
                    cur_time = (new Date()).getTime();
                }
                return -1; // timeout case!
            }
			*/
			{	// now block until timeout
				int sleepLength = 10;
                int passes = readTimeout / sleepLength;
                for (int i = 0; i < passes; i++)
				{
                    Time.Sleep(sleepLength);
                    if (inStream.available() > 0)
					{
                        inStream.read(input_byte);
                        if (DEBUG_MORE)
							System.err.println("ReadByte: "+ByteUtil.Unsign(input_byte[0]));
                        return ByteUtil.Unsign(input_byte[0]);
                    }
                }
                return -1; // timeout case!
            }
		}
		catch (Exception e)
		{
            System.err.println("Exception in NetPort.java on available(): " + e);
            return -1;
		}
    }
    
    public int SetReadTimeout(int newTimeout)
	{
        readTimeout = newTimeout;
        return 0;
    }
    
    public void Flush()
	{
		synchronized (inStream)
		{
			try
			{
				int n = inStream.available();
				if (DEBUG_MORE)
				{
					if (n > 0)
					{
						byte[] b = new byte[1024];
						int l = inStream.read(b);
						for (int i = 0; i < l; i++)
							System.err.println("Flushing byte: " + ByteUtil.Unsign(b[i]));
					}
				}
				else
					inStream.skip(n);
			}
			catch (Exception e)
			{
				System.err.println("Error in NetPort FlushBuffer(): " + e);
			}
		}
	}
    
    public void Flush(String s)
	{
        if (s != null && !s.equals(""))
            SendString(s);
		Flush();
    }

	public void SendQuit()
	{
		synchronized (outStream)
		{
			try
			{
				outStream.write(quit);
			}
			catch (IOException e)
			{
				System.err.println("outStream.write: IO exception " + e);
			}
		}
	}

	public boolean StartLogging(OutputStream out)
	{
		log = out;
		time = Time.Current();
		logging = true;

		return true;
	}

	public boolean StopLogging()
	{
		logging = false;
		return true;
	}

	public boolean ReplayLog(InputStream in)
	{
		try
		{
			byte[] timeDiff = new byte[8];
	
			while (in.read(timeDiff, 0, 8) == 8) // Go till we hit end of file
			{
				Time.Sleep(ByteUtil.BuildLong(timeDiff));
				int length;
				length = in.read();
				byte[] command = new byte[length];
				in.read(command, 0, length);
				SendBytes(command);
			}
			
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}
}