/*
 * TrikebotCommand.java
 *
 * Created on April 12, 2002, 11:33 AM
 */

/*

See "iPaq Protocol.pdf" for a description of how command and response packets work.
 
*/

public class TrikebotCommand
{
	private final static byte SET_PARAMETERS		= (byte) 'P';
	private final static byte RESET_INTEGRATION		= (byte) 'I';
	private final static byte COLOR_TRACK			= (byte) 'T';
	private final static byte COLOR_TRACK_NO_TILT	= (byte) 'U';
	private final static byte COLOR_TRACK_NO_PAN	= (byte) 'V';
	private final static byte COLOR_TRACK_NO_MOVE	= (byte) 'A';
	private final static byte WINDOW_TRACK			= (byte) 'W';
	private final static byte WINDOW_TRACK_NO_TILT	= (byte) 'X';
	private final static byte WINDOW_TRACK_NO_PAN	= (byte) 'Y';
	private final static byte WINDOW_TRACK_NO_MOVE	= (byte) 'Z';
	private final static byte STOP_TRACK			= (byte) 'N';
	private final static byte SAFETY				= (byte) 'S';
	private final static byte DUMP_FRAME			= (byte) 'D';
	private final static byte CMUCAM				= (byte) 'C';
	private final static byte BRAINSTEM				= (byte) 'B';

	private byte[] packet;

    /** Creates new TrikebotCommand */
    public TrikebotCommand()
	{
        // Do nothing
    }

	private void Length(int l) // Length of the packet, not including starting and ending delimiters
	{
		packet = new byte[l];
	}

	private void Type(byte t)
	{
		packet[0] = t;
	}

	public byte[] LastCommand()
	{
		return packet;
	}
	
    public byte[] SetParameters(boolean tmset, int time, boolean vset, int vel, boolean tset, int turn,
							   boolean cpset, int pan, boolean ctset, int tilt)
	{
		Length(8);
		Type(SET_PARAMETERS);
		
		packet[1] = (byte) time;
        packet[2] = (byte) vel;
        packet[3] = (byte) turn;
        packet[4] = (byte) pan;
        packet[5] = (byte) tilt;

		byte mask = 0;
		if (tmset)
			mask |= 1;
		if (vset)
			mask |= 2;
		if (tset)
			mask |= 4;
		if (cpset)
			mask |= 8;
		if (ctset)
			mask |= 16;
		packet[6] = mask;

		Checksum.Set(packet);
		
		return packet;
    }

	public byte[] KillMotor()
	{
		SetParameters(false, 0, false, 0, false, 0, false, 0, false, 0);
		packet[6] |= 32;
		Checksum.Set(packet);
		return packet;
	}

	public byte[] ResetRobot()
	{
		SetParameters(false, 0, false, 0, false, 0, false, 0, false, 0);
		packet[6] |= 64;
		Checksum.Set(packet);
		return packet;
	}
	
	public byte[] ResetIntegration(boolean xset, int x, boolean yset, int y, boolean thset, int th)
	{
		Length(9);
		Type(RESET_INTEGRATION);

		packet[1] = ByteUtil.Hi(x);
        packet[2] = ByteUtil.Lo(x);
		packet[3] = ByteUtil.Hi(y);
        packet[4] = ByteUtil.Lo(y);
		packet[5] = ByteUtil.Hi(th);
        packet[6] = ByteUtil.Lo(th);

		byte mask = 0;
		if (xset)
			mask |= 1;
		if (yset)
			mask |= 2;
		if (thset)
			mask |= 4;
		packet[7] = mask;

		Checksum.Set(packet);

		return packet;
    }

	public byte[] ActiveColorTracking(int rmin, int rmax, int gmin, int gmax, int bmin, int bmax)
	{
		Length(8);
		Type(COLOR_TRACK);

		packet[1] = (byte) rmin;
        packet[2] = (byte) rmax;
		packet[3] = (byte) gmin;
        packet[4] = (byte) gmax;
		packet[5] = (byte) bmin;
        packet[6] = (byte) bmax;

		Checksum.Set(packet);

		return packet;
    }

	public byte[] ActiveColorTrackingNoTilt(int rmin, int rmax, int gmin, int gmax, int bmin, int bmax)
	{
		ActiveColorTracking(rmin, rmax, gmin, gmax, bmin, bmax);
		Type(COLOR_TRACK_NO_TILT);
		Checksum.Set(packet);
		return packet;
	}

	public byte[] ActiveColorTrackingNoPan(int rmin, int rmax, int gmin, int gmax, int bmin, int bmax)
	{
		ActiveColorTracking(rmin, rmax, gmin, gmax, bmin, bmax);
		Type(COLOR_TRACK_NO_PAN);
		Checksum.Set(packet);
		return packet;
	}

	public byte[] ActiveColorTrackingNoMove(int rmin, int rmax, int gmin, int gmax, int bmin, int bmax)
	{
		ActiveColorTracking(rmin, rmax, gmin, gmax, bmin, bmax);
		Type(COLOR_TRACK_NO_MOVE);
		Checksum.Set(packet);
		return packet;
	}

	public byte[] ActiveWindowTracking()
	{
		Length(2);
		Type(WINDOW_TRACK);
		Checksum.Set(packet);
		return packet;
    }

	public byte[] ActiveWindowTrackingNoTilt()
	{
		Length(2);
		Type(WINDOW_TRACK_NO_TILT);
		Checksum.Set(packet);
		return packet;
    }

	public byte[] ActiveWindowTrackingNoPan()
	{
		Length(2);
		Type(WINDOW_TRACK_NO_PAN);
		Checksum.Set(packet);
		return packet;
    }

	public byte[] ActiveWindowTrackingNoMove()
	{
		Length(2);
		Type(WINDOW_TRACK_NO_MOVE);
		Checksum.Set(packet);
		return packet;
    }

	public byte[] StopActiveTracking()
	{
		Length(2);
		Type(STOP_TRACK);
		Checksum.Set(packet);
		return packet;
    }

	public byte[] Safety(byte reading)
	{
		Length(3);
		Type(SAFETY);

		packet[1] = reading;

		Checksum.Set(packet);

		return packet;
    }
	
	public byte[] DumpFrame()
	{
		Length(2);
		Type(DUMP_FRAME);
		Checksum.Set(packet);
		return packet;
    }
	
	public byte[] CMUcam(String s)
	{
		int l = s.length();
		Length(l + 3);
		Type(CMUCAM);
		packet[1] = (byte) l;
		for (int i = 0; i < l; i++)
			packet[i+2] = s.getBytes()[i];
		Checksum.Set(packet);
		return packet;
    }

	public byte[] BrainStem(byte[] b)
	{
		int l = b.length;
		Length(l + 3);
		Type(BRAINSTEM);
		packet[1] = (byte) l;
		for (int i = 0; i < l; i++)
			packet[i+2] = b[i];
		Checksum.Set(packet);
		return packet;
    }
}
