//
//  TrikebotState.java
//  Trikebot
//
//  Created by Andrs Santiago Prez-Bergquist on Thu Jun 06 2002.
//  Copyright (c) 2002. All rights reserved.
//

/*

See "iPaq Protocol.pdf" for a description of how command and response packets work.

*/

public class TrikebotState extends java.lang.Object
{
	private int status;
	private int range;
	private int pan;
	private int tilt;
	private int wheelAngle;
	private int wheelVoltage;
	private int wheelCurrent;
	private int integratedX;
	private int integratedY;
	private int integratedTH;
	
	public final static int FULL_PACKET_LENGTH = 13;
	public final static int MINI_PACKET_LENGTH = 2;

	public final static int SUCCESS					=   0;
	public final static int IPAQ_INVALID_CHECKSUM	=   1;
	public final static int IPAQ_UNKNOWN_PACKET_TYPE=   2;
	public final static int CAMERA_TIMEOUT			=  10;
	public final static int CAMERA_SPEW				=  11;
	public final static int BRAINSTEM_TIMEOUT		=  20;
	public final static int NO_RESPONSE				= 100;
	public final static int INVALID_PACKET_LENGTH	= 101;
	public final static int INVALID_CHECKSUM 		= 102;
	
	/** Creates new TrikebotResponse */
	public TrikebotState()
	{
		status			= 0;
		range			= 0;
		pan				= 0;
		tilt			= 0;
		wheelAngle		= 0;
		wheelVoltage	= 0;
		wheelCurrent	= 0;
		integratedX		= 0;
		integratedY		= 0;
		integratedTH	= 0;
	}
	
	public boolean ParseFullPacket (byte[] packet)
	{
		if (packet.length == 0)
		{
			status = NO_RESPONSE;
			System.err.println ("No response, expected "+FULL_PACKET_LENGTH+" bytes!");
			return false;
		}
		if (packet.length != FULL_PACKET_LENGTH)
		{
			status = INVALID_PACKET_LENGTH;
			System.err.println ("Invalid packet; got "+packet.length+" bytes, expected "+FULL_PACKET_LENGTH+"!");
			return false;
		}
		if (!Checksum.IsValid(packet))
		{
			status = INVALID_CHECKSUM;
			System.err.println ("Invalid checksum; read "+packet[packet.length-1]+", computed "+Checksum.Compute(packet)+"!");
			return false;
		}
		
		status			= packet[0];
		range			= ByteUtil.Unsign(packet[1]);
		wheelVoltage	= ByteUtil.AssembleSigned(packet[2], packet[3]);
		wheelCurrent	= ByteUtil.AssembleSigned(packet[4], packet[5]);
		integratedX		= ByteUtil.AssembleSigned(packet[6], packet[7]);
		integratedY		= ByteUtil.AssembleSigned(packet[8], packet[9]);
		integratedTH	= ByteUtil.AssembleSigned(packet[10], packet[11]);

		return true;
	}

	public boolean ParseMiniPacket (byte[] packet)
	{
		if (packet.length == 0)
		{
			status = NO_RESPONSE;
			System.err.println ("No response, expected "+MINI_PACKET_LENGTH+" bytes!");
			return false;
		}
		if (packet.length != MINI_PACKET_LENGTH)
		{
			status = INVALID_PACKET_LENGTH;
			System.err.println ("Invalid packet; got "+packet.length+" bytes, expected "+MINI_PACKET_LENGTH+"!");
			return false;
		}
		if (!Checksum.IsValid(packet))
		{
			status = INVALID_CHECKSUM;
			System.err.println ("Invalid checksum; read "+packet[packet.length-1]+", computed "+Checksum.Compute(packet)+"!");
			return false;
		}

		status = packet[0];

		return true;
	}

	public String ParseStringPacket (byte[] packet)
	{
		if (packet.length == 0)
		{
			status = NO_RESPONSE;
			System.err.println ("No response, expected "+FULL_PACKET_LENGTH+" bytes!");
			return null;
		}
		int length = ByteUtil.Unsign(packet[1]);
		if (packet.length != length+3)
		{
			status = INVALID_PACKET_LENGTH;
			System.err.println ("Invalid packet; got "+packet.length+" bytes, expected "+(length+3)+"!");
			return null;
		}
		if (!Checksum.IsValid(packet))
		{
			status = INVALID_CHECKSUM;
			System.err.println ("Invalid checksum; read "+packet[packet.length-1]+", computed "+Checksum.Compute(packet)+"!");
			return null;
		}

		status = packet[0];
		
		byte[] b = new byte[length];
		for (int i = 0; i < length; i++)
			b[i] = (byte) packet[i+2];

		return new String(b);
	}

	// Accessors
	public int Status()			{ return status; }
	public int Range()			{ return range; }
	public int Pan()			{ return pan; }
	public int Tilt()			{ return tilt; }
	public int WheelAngle()		{ return wheelAngle; }
	public int WheelVelocity()	{ return wheelVoltage; }
	public int WheelCurrent()	{ return wheelCurrent; }
	public int IntegratedX()	{ return integratedX; }
	public int IntegratedY()	{ return integratedY; }
	public int IntegratedTH()	{ return integratedTH; }

	public int SetPan(int p)		 { pan = p;			return p; }
	public int SetTilt(int t)		 { tilt = t;		return t; }
	public int SetWheelAngle(int wa) { wheelAngle = wa;	return wheelAngle; }
}
